; ##############################################################################
; ##############################################################################
; ##############################################################################
;  CPU            SCL															
;1.000.000	=	 27.778															
;2.000.000	=	 55.556															
;4.000.000	=	111.111															
;8.000.000	=	222.222															
;16.000.000	=	444.444															
TWI_INI:
.equ TWI_BIT_RATE				=255				; 10	; Bit-Rate							
.equ TWI_PRESCALER				=1				; 0		; Vorteiler: 0=1 1=4 2=16 3=64		
	
	
	; TWI TWBR 				Bit RATE Register	
	ldi temp, TWI_BIT_RATE
	out TWBR, temp
	; TWI TWSR 				STATUS-Register		
	ldi temp, TWI_PRESCALER
	out TWSR, temp
	; TWI TWAR (nur SLAVE) 	Adress-Register		
	clr temp
	out TWAR,temp
	; TWI TWCR 				Control-REGISTER	
	ldi temp, 1<<TWINT|1<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|1<<TWEN|0<<TWIE
	out TWCR,temp

	sbi PORTC,4	; Pull Up on	
	sbi PORTC,5

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TWI_MT_START_TEMP1:
	clr ERROR
	WDR
	; START								
	ldi temp, (1<<TWINT)|(1<<TWSTA)|(1<<TWEN)
	out TWCR, temp	
	; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_wait									
	; Check TWSR						
	in temp,TWSR													
	andi temp, 0xF8
	cpi temp,  0x08							; 0x08 START erfolgreich
	breq TWI_MT_START_w
	cpi temp,  0x10							; 0x10 (repeated START)
	breq TWI_MT_START_w
	; ERROR		
	ldi temp,(ERROR_TWI_MT_START)
	mov ERROR, temp
	LED_ROT_ON
	ret
;-------------------------------------------------------------------------------
TWI_MT_START_w:
	; Load SLA + "W"  (W=0) 			
	LSL  temp1								; 1 x links
	out TWDR, temp1
	; senden - Clear TWINT 				
	ldi temp, (1<<TWINT) | (1<<TWEN)							
	out TWCR, temp
	; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_wait									
	; Check TWSR						
	in temp,TWSR												
	andi temp, 0xF8
	cpi temp,  0x18							; 0x18
	breq TWI_MT_START_OK
	; ERROR		
	ldi temp,(ERROR_TWI_MT_SLA)
	mov ERROR, temp
	LED_ROT_ON
	ret
;-------------------------------------------------------------------------------
TWI_MT_START_OK:

ret
; #############################################################################
; #############################################################################
; #############################################################################
TWI_MT_TX_TEMP1:
	WDR
	tst ERROR							; Fehler =0 ?	
	breq TWI_MT_TX_w					; ja			
	ret									; zurck		
;-------------------------------------------------------------------------------
TWI_MT_TX_w:
	; 	Load DATA						
	out TWDR, temp1							
	; senden - Clear TWINT 				
	ldi temp, (1<<TWINT) | (1<<TWEN)							
	out TWCR, temp
	; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_wait									
	; Check TWSR						
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x28	; 0x28
	breq TWI_MT_TX_ok
	; ERROR								
	ldi temp,(ERROR_TWI_MT_DATA)
	mov ERROR, temp
	LED_ROT_ON
	ret
;-------------------------------------------------------------------------------
TWI_MT_TX_ok:

ret
; #############################################################################
; #############################################################################
; #############################################################################
TWI_MASTER_RX_START:
	WDR
	clr ERROR
	; SRAM lschen						
	STS	(adr_TWI_BEREICH_READ+0),NULL
	; START 											
	ldi temp, (1<<TWINT | 1<<TWEA | 1<<TWSTA | 1<<TWEN )
	out TWCR, temp	
	; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_wait
	; Check TWSR 										
	in temp,TWSR		
	andi temp, 0xF8
	cpi temp,  0x08 								; 0x08 START erfolgreich
	breq TWI_MASTER_RX_w
	cpi temp,  0x10									; 0x10 (repeated START)
	breq TWI_MASTER_RX_w
	; ERROR												
	ldi temp,(ERROR_TWI_MR_START)
	mov ERROR, temp
	LED_ROT_ON
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RX_w:
	; Load SLA +"R" (R=1)								
	lsl  temp1	; 1 x links
	ori temp1, 0b00000001 
	out TWDR, temp1
	; senden - Clear TWINT 								
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	out TWCR, temp
	; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_wait
	; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x40 	;SLA+R gesendet; ACK empfangen
	breq TWI_MASTER_RX_ok
	; ERROR						
	ldi temp,(ERROR_TWI_MR_SLA)
	mov ERROR, temp
	LED_ROT_ON
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RX_oK:

ret
; #############################################################################
; #############################################################################
; #############################################################################
;																				
; Achtung, diese Routine verwenden, wenn 										
; mehrere! Zeichen empfangen werden sollen										
;																				
; OUTPUT: temp1																	
;																				
TWI_MASTER_RX_DATA:
	; Fehlerprfung										
	tst ERROR
	breq TWI_MASTER_RX_DATA_w
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RX_DATA_w:
	; senden "ACK" (TWEA=1) fr mehrere Daten-Bytes 	
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	out TWCR, temp
	; TWINT abfragen									
	rcall TWI_TWINT_wait
	; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x50	; Data Byte empfangen- ACK zurckgesendet
	breq TWI_MASTER_RX_DATA_OK
	; ERROR
	clr temp1
	ldi temp,(ERROR_TWI_MR_DATA)
	mov ERROR, temp	
	LED_ROT_ON
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RX_DATA_OK:
	; DATEN einlesen							
	in temp1, TWDR						
ret
; #############################################################################
; #############################################################################
; #############################################################################
;																				
; Achtung, diese Routine verwenden, wenn 										
; nur ein! Zeichen empfangen werden soll, oder wenn								
; das letzte! Zeichen empfangen werden soll										
;																				
; OUTPUT: temp1																	
;																				
TWI_MASTER_RX_DATA_LAST_BYTES:
	tst ERROR
	breq TWI_MASTER_RX_DATA_LAST_BYTES_w
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RX_DATA_LAST_BYTES_w:
	; senden "NOT ACK" (TWEA=0) fr letztes bzw fr ein Daten-Byte
	ldi temp, (1<<TWINT | 0<<TWEA  | 1<<TWEN )
	out TWCR, temp
	; TWINT abfragen									
	rcall TWI_TWINT_wait
	; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi  temp, 0x58	; letztes Data Byte empfangen-NOT ACK zurckgesendet
	breq TWI_MASTER_RX_DATA_LAST_BYTES_OK	; ERROR
	; ERROR
	clr temp1
	ldi temp,(ERROR_TWI_MR_DATAEND)
	mov ERROR, temp	
	LED_ROT_ON
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RX_DATA_LAST_BYTES_OK:
	; DATEN einlesen		
	in temp1, TWDR
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; STOP- Routine																   
; 																			   
TWI_STOP:
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)|0<<TWIE	; STOP								
	out TWCR, temp	
	rcall WAIT_1ms
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; warten, bis das TWINT Flag gesetzt ist									   
; 																			   
TWI_TWINT_wait:
	LED_GELB_ON
	in temp, TWCR
	sbrs temp, TWINT	; nchsten Befehl berspringen, wenn TWINT FLAG  gesetzt ist
	rjmp TWI_TWINT_wait	
	LED_GELB_OFF
ret
; #############################################################################
; #############################################################################
; #############################################################################
